@@ -99,9 +99,46 @@ dectalk_synthesis_driver_from_servant (PortableServer_Servant *servant)
 static gboolean
 dectalk_idle (gpointer data)
 {
+	gchar  *converted_text;
+	GError *gerror = NULL;
+	
 	DectalkSynthesisDriver *d = DECTALK_SYNTHESIS_DRIVER (data);
 
-	TextToSpeechSpeak (d->handle, d->text, TTS_FORCE);
+	/* After some conversation with the DECtalk engineers, it is
+	 * unclear about whether DECtalk wants strings from code page 
+	 * 850 or ISO8859-1.  Apparently there are notes in the DECtalk
+	 * sources that talk about code page 850, but the general 
+	 * consensus seems to be that ISO8859-1 is really what is 
+	 * expected.  
+	 *
+	 * We're getting strings in UTF-8, so we convert.  This will 
+	 * help prevent DECtalk from crashing (see bug 346357 at
+	 * http://bugzilla.gnome.org/show_bug.cgi?id=347357).  In
+	 * addition, since DECtalk has been shown to crash if we give
+	 * it bad strings, we will not send anything to speak if the
+	 * conversion failed.
+	 */
+	converted_text = g_convert_with_fallback (
+	        d->text, 
+		-1,          /* d->text is a null-terminated string */
+		"ISO8859-1", /* what DECtalk wants                  */
+		"UTF-8",     /* what we are getting                 */
+		" ",         /* the fallback hint                   */
+		NULL, NULL,  /* bytes read and written              */
+		&gerror);    /* what went wrong, if anything        */
+
+	if (!converted_text) {
+	        printf ("STRING CONVERSION FAILED: (string=%s) (error=%s)!\n", 
+		        d->text, gerror->message);
+        } else {
+#if 0
+	        printf ("STRING CONVERSION: (from=%s)\n", d->text); 
+	        printf ("STRING CONVERSION:   (to=%s)\n", converted_text); 
+#endif
+		TextToSpeechSpeak (d->handle, converted_text, TTS_FORCE);
+		g_free (converted_text);
+        }
+
 	g_free (d->text);
 	d->text = NULL;
 	d->text_idle = 0;
@@ -193,10 +230,14 @@ dectalk_callback (LONG lParam1,
 	DectalkSynthesisDriver *d = DECTALK_SYNTHESIS_DRIVER(dwCallbackParameter);
 	index_queue_entry *e = NULL;
 
+	if (!d) {
+	        return;
+	}
+
 	switch (uiMsg) {
 	case TTS_MSG_INDEX_MARK:
-		if (d && d->index_queue) {
-			g_mutex_lock (d->mutex);
+	        g_mutex_lock (d->mutex);
+		if (d->index_queue) {
 			do {
 			    e = (index_queue_entry *) d->index_queue->data;
 			    if (lParam2 >= e->id)
@@ -205,8 +246,8 @@ dectalk_callback (LONG lParam1,
 				d->post_queue = g_slist_append (d->post_queue, e);
 			    }
 			}while (lParam2 >= e->id && d->index_queue);
-			g_mutex_unlock (d->mutex);
 		};
+		g_mutex_unlock (d->mutex);
 		break;
 	case TTS_MSG_STATUS:
 		if (lParam1 == TTS_AUDIO_PLAY_STOP) {
