@@ -27,7 +27,7 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <glib/gmain.h>
+#include <glib.h>
 #include <libbonobo.h>
 #include <gnome-speech/gnome-speech.h>
 #include "callback.h"
